@@ -1,18 +1,14 @@
 # -*- test-case-name: twisted.web.test.test_http -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 HyperText Transfer Protocol implementation.
 
-This is used by twisted.web.
-
-Future Plans:
- - HTTP client support will at some point be refactored to support HTTP/1.1.
- - Accept chunked data from clients in server.
- - Other missing HTTP features from the RFC.
-
-Maintainer: Itamar Shtull-Trauring
+This is the basic server-side protocol implementation used by the Twisted
+Web server.  It can parse HTTP 1.0 requests and supports many HTTP 1.1
+features as well.  Additionally, some functionality implemented here is
+also useful for HTTP clients (such as the chunked encoding parser).
 """
 
 # system imports
@@ -35,10 +31,7 @@ from twisted.internet import interfaces, reactor, protocol, address
 from twisted.internet.defer import Deferred
 from twisted.protocols import policies, basic
 from twisted.python import log
-try: # try importing the fast, C version
-    from twisted.protocols._c_urlarg import unquote
-except ImportError:
-    from urllib import unquote
+from urllib import unquote
 
 from twisted.web.http_headers import _DictHeaders, Headers
 
@@ -239,33 +232,6 @@ def datetimeToLogString(msSinceEpoch=None):
         hh, mm, ss)
     return s
 
-
-# a hack so we don't need to recalculate log datetime every hit,
-# at the price of a small, unimportant, inaccuracy.
-_logDateTime = None
-_logDateTimeUsers = 0
-_resetLogDateTimeID = None
-
-def _resetLogDateTime():
-    global _logDateTime
-    global _resetLogDateTime
-    global _resetLogDateTimeID
-    _logDateTime = datetimeToLogString()
-    _resetLogDateTimeID = reactor.callLater(1, _resetLogDateTime)
-
-def _logDateTimeStart():
-    global _logDateTimeUsers
-    if not _logDateTimeUsers:
-        _resetLogDateTime()
-    _logDateTimeUsers += 1
-
-def _logDateTimeStop():
-    global _logDateTimeUsers
-    _logDateTimeUsers -= 1;
-    if (not _logDateTimeUsers and _resetLogDateTimeID
-        and _resetLogDateTimeID.active()):
-        _resetLogDateTimeID.cancel()
-
 def timegm(year, month, day, hour, minute, second):
     """
     Convert time tuple in GMT to seconds since epoch, GMT
@@ -919,6 +885,7 @@ class Request:
         if not self.queued:
             self._cleanup()
 
+
     def write(self, data):
         """
         Write some data as a result of an HTTP request.  The first
@@ -927,6 +894,9 @@ class Request:
         @type data: C{str}
         @param data: Some bytes to be sent as part of the response body.
         """
+        if self.finished:
+            raise RuntimeError('Request.write called on a request after '
+                               'Request.finish was called.')
         if not self.startedWriting:
             self.startedWriting = 1
             version = self.clientproto
@@ -1176,8 +1146,16 @@ class Request:
         @param ssl: A flag which, if C{True}, indicates that the request is
             considered secure (if C{True}, L{isSecure} will return C{True}).
         """
-        self._forceSSL = ssl
-        self.requestHeaders.setRawHeaders("host", [host])
+        self._forceSSL = ssl # set first so isSecure will work
+        if self.isSecure():
+            default = 443
+        else:
+            default = 80
+        if port == default:
+            hostHeader = host
+        else:
+            hostHeader = '%s:%d' % (host, port)
+        self.requestHeaders.setRawHeaders("host", [hostHeader])
         self.host = address.IPv4Address("TCP", host, port)
 
 
@@ -1728,6 +1706,14 @@ class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin):
 class HTTPFactory(protocol.ServerFactory):
     """
     Factory for HTTP server.
+
+    @ivar _logDateTime: A cached datetime string for log messages, updated by
+        C{_logDateTimeCall}.
+    @type _logDateTime: L{str}
+
+    @ivar _logDateTimeCall: A delayed call for the next update to the cached log
+        datetime string.
+    @type _logDateTimeCall: L{IDelayedCall} provided
     """
 
     protocol = HTTPChannel
@@ -1742,6 +1728,19 @@ class HTTPFactory(protocol.ServerFactory):
         self.logPath = logPath
         self.timeOut = timeout
 
+        # For storing the cached log datetime and the callback to update it
+        self._logDateTime = None
+        self._logDateTimeCall = None
+
+
+    def _updateLogDateTime(self):
+        """
+        Update log datetime periodically, so we aren't always recalculating it.
+        """
+        self._logDateTime = datetimeToLogString()
+        self._logDateTimeCall = reactor.callLater(1, self._updateLogDateTime)
+
+
     def buildProtocol(self, addr):
         p = protocol.ServerFactory.buildProtocol(self, addr)
         # timeOut needs to be on the Protocol instance cause
@@ -1749,19 +1748,30 @@ class HTTPFactory(protocol.ServerFactory):
         p.timeOut = self.timeOut
         return p
 
+
     def startFactory(self):
-        _logDateTimeStart()
+        """
+        Set up request logging if necessary.
+        """
+        if self._logDateTimeCall is None:
+            self._updateLogDateTime()
+
         if self.logPath:
             self.logFile = self._openLogFile(self.logPath)
         else:
             self.logFile = log.logfile
 
+
     def stopFactory(self):
         if hasattr(self, "logFile"):
             if self.logFile != log.logfile:
                 self.logFile.close()
             del self.logFile
-        _logDateTimeStop()
+
+        if self._logDateTimeCall is not None and self._logDateTimeCall.active():
+            self._logDateTimeCall.cancel()
+            self._logDateTimeCall = None
+
 
     def _openLogFile(self, path):
         """
@@ -1786,7 +1796,7 @@ class HTTPFactory(protocol.ServerFactory):
             line = '%s - - %s "%s" %d %s "%s" "%s"\n' % (
                 request.getClientIP(),
                 # request.getUser() or "-", # the remote user is almost never important
-                _logDateTime,
+                self._logDateTime,
                 '%s %s %s' % (self._escape(request.method),
                               self._escape(request.uri),
                               self._escape(request.clientproto)),
