@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -51,6 +51,7 @@ demo webserver that has the Test class from twisted.web.demo in it."""
         self['indexes'] = []
         self['root'] = None
 
+
     def opt_index(self, indexName):
         """
         Add the name of a file used to check for directory indexes.
@@ -60,6 +61,7 @@ demo webserver that has the Test class from twisted.web.demo in it."""
 
     opt_i = opt_index
 
+
     def opt_user(self):
         """
         Makes a server with ~/public_html and ~/.twistd-web-pb support for
@@ -69,6 +71,7 @@ demo webserver that has the Test class from twisted.web.demo in it."""
 
     opt_u = opt_user
 
+
     def opt_path(self, path):
         """
         <path> is either a specific file or a directory to be set as the root
@@ -76,23 +79,31 @@ demo webserver that has the Test class from twisted.web.demo in it."""
         php3, epy, or rpy files or any other files that you want to be served
         up raw.
         """
-        def trp(*args, **kwargs):
-            # Help avoid actually importing twisted.web.trp until it is really
-            # needed.  This avoids getting a deprecation warning if you're not
-            # using deprecated functionality.
-            from twisted.web import trp
-            return trp.ResourceUnpickler(*args, **kwargs)
+
+        def php3(*args, **kwargs):
+            # Help avoid actually importing twisted.web.twcgi.PHP3Script until
+            # it is really needed. This avoids getting a deprecation warning if
+            # you're not using deprecated functionality.
+            from twisted.web.twcgi import PHP3Script
+            return PHP3Script(*args, **kwargs)
+
+        def php(*args, **kwargs):
+            # Help avoid actually importing twisted.web.twcgi.PHPScript until it
+            # is really needed. This avoids getting a deprecation warning if
+            # you're not using deprecated functionality.
+            from twisted.web.twcgi import PHPScript
+            return PHPScript(*args, **kwargs)
 
         self['root'] = static.File(os.path.abspath(path))
         self['root'].processors = {
             '.cgi': twcgi.CGIScript,
-            '.php3': twcgi.PHP3Script,
-            '.php': twcgi.PHPScript,
+            '.php3': php3,
+            '.php': php,
             '.epy': script.PythonScript,
             '.rpy': script.ResourceScript,
-            '.trp': trp,
             }
 
+
     def opt_processor(self, proc):
         """
         `ext=class' where `class' is added as a Processor for files ending
@@ -103,6 +114,7 @@ demo webserver that has the Test class from twisted.web.demo in it."""
         ext, klass = proc.split('=', 1)
         self['root'].processors[ext] = reflect.namedClass(klass)
 
+
     def opt_class(self, className):
         """
         Create a Resource subclass with a zero-argument constructor.
@@ -152,6 +164,7 @@ demo webserver that has the Test class from twisted.web.demo in it."""
                                    "after --path.")
         self['root'].ignoreExt('*')
 
+
     def opt_ignore_ext(self, ext):
         """
         Specify an extension to ignore.  These will be processed in order.
@@ -161,6 +174,7 @@ demo webserver that has the Test class from twisted.web.demo in it."""
                                    "after --path.")
         self['root'].ignoreExt(ext)
 
+
     def postOptions(self):
         """
         Set up conditional defaults and check for dependencies.
