@@ -1,4 +1,4 @@
-# Copyright (c) 2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -10,6 +10,7 @@ __metaclass__ = type
 from zope.interface import implements
 from zope.interface.verify import verifyObject
 
+from twisted.python import log
 from twisted.python.failure import Failure
 from twisted.internet.interfaces import IConsumer, IPushProducer
 from twisted.internet.error import ConnectionDone
@@ -20,7 +21,10 @@ from twisted.test.proto_helpers import StringTransport, AccumulatingProtocol
 from twisted.web._newclient import UNKNOWN_LENGTH, STATUS, HEADER, BODY, DONE
 from twisted.web._newclient import Request, Response, HTTPParser, HTTPClientParser
 from twisted.web._newclient import BadResponseVersion, ParseError, HTTP11ClientProtocol
-from twisted.web._newclient import ChunkedEncoder, RequestGenerationFailed, RequestTransmissionFailed, ResponseFailed, WrongBodyLength, RequestNotSent
+from twisted.web._newclient import ChunkedEncoder, RequestGenerationFailed
+from twisted.web._newclient import RequestTransmissionFailed, ResponseFailed
+from twisted.web._newclient import WrongBodyLength, RequestNotSent
+from twisted.web._newclient import ConnectionAborted
 from twisted.web._newclient import BadHeaders, ResponseDone, PotentialDataLoss, ExcessWrite
 from twisted.web._newclient import TransportProxyProducer, LengthEnforcingConsumer, makeStatefulDispatcher
 from twisted.web.http_headers import Headers
@@ -74,7 +78,8 @@ def assertWrapperExceptionTypes(self, deferred, mainType, reasonTypes):
     def cbFailed(err):
         for reason, type in zip(err.reasons, reasonTypes):
             reason.trap(type)
-        self.assertEqual(len(err.reasons), len(reasonTypes))
+        self.assertEqual(len(err.reasons), len(reasonTypes),
+                         "len(%s) != len(%s)" % (err.reasons, reasonTypes))
         return err
     d = self.assertFailure(deferred, mainType)
     d.addCallback(cbFailed)
@@ -150,11 +155,13 @@ class MakeStatefulDispatcherTests(TestCase):
 
 
 
-class HTTPParserTests(TestCase):
+class _HTTPParserTests(object):
     """
-    Tests for L{HTTPParser} which is responsible for the bulk of the task of
-    parsing HTTP bytes.
+    Base test class for L{HTTPParser} which is responsible for the bulk of
+    the task of parsing HTTP bytes.
     """
+    sep = None
+
     def test_statusCallback(self):
         """
         L{HTTPParser} calls its C{statusReceived} method when it receives a
@@ -165,7 +172,7 @@ class HTTPParserTests(TestCase):
         protocol.statusReceived = status.append
         protocol.makeConnection(StringTransport())
         self.assertEqual(protocol.state, STATUS)
-        protocol.dataReceived('HTTP/1.1 200 OK\r\n')
+        protocol.dataReceived('HTTP/1.1 200 OK' + self.sep)
         self.assertEqual(status, ['HTTP/1.1 200 OK'])
         self.assertEqual(protocol.state, HEADER)
 
@@ -175,7 +182,7 @@ class HTTPParserTests(TestCase):
         protocol = HTTPParser()
         protocol.headerReceived = header.__setitem__
         protocol.makeConnection(StringTransport())
-        protocol.dataReceived('HTTP/1.1 200 OK\r\n')
+        protocol.dataReceived('HTTP/1.1 200 OK' + self.sep)
         return header, protocol
 
 
@@ -185,10 +192,10 @@ class HTTPParserTests(TestCase):
         header.
         """
         header, protocol = self._headerTestSetup()
-        protocol.dataReceived('X-Foo:bar\r\n')
+        protocol.dataReceived('X-Foo:bar' + self.sep)
         # Cannot tell it's not a continue header until the next line arrives
         # and is not a continuation
-        protocol.dataReceived('\r\n')
+        protocol.dataReceived(self.sep)
         self.assertEqual(header, {'X-Foo': 'bar'})
         self.assertEqual(protocol.state, BODY)
 
@@ -199,10 +206,10 @@ class HTTPParserTests(TestCase):
         C{headerReceived} with the entire value once it is received.
         """
         header, protocol = self._headerTestSetup()
-        protocol.dataReceived('X-Foo: bar\r\n')
-        protocol.dataReceived(' baz\r\n')
-        protocol.dataReceived('\tquux\r\n')
-        protocol.dataReceived('\r\n')
+        protocol.dataReceived('X-Foo: bar' + self.sep)
+        protocol.dataReceived(' baz' + self.sep)
+        protocol.dataReceived('\tquux' + self.sep)
+        protocol.dataReceived(self.sep)
         self.assertEqual(header, {'X-Foo': 'bar baz\tquux'})
         self.assertEqual(protocol.state, BODY)
 
@@ -213,10 +220,10 @@ class HTTPParserTests(TestCase):
         value passed to the C{headerReceived} callback.
         """
         header, protocol = self._headerTestSetup()
-        value = ' \t \r\n bar \t\r\n \t\r\n'
+        value = ' \t %(sep)s bar \t%(sep)s \t%(sep)s' % dict(sep=self.sep)
         protocol.dataReceived('X-Bar:' + value)
         protocol.dataReceived('X-Foo:' + value)
-        protocol.dataReceived('\r\n')
+        protocol.dataReceived(self.sep)
         self.assertEqual(header, {'X-Foo': 'bar',
                                   'X-Bar': 'bar'})
 
@@ -232,7 +239,7 @@ class HTTPParserTests(TestCase):
             called.append(protocol.state)
             protocol.state = STATUS
         protocol.allHeadersReceived = allHeadersReceived
-        protocol.dataReceived('\r\n')
+        protocol.dataReceived(self.sep)
         self.assertEqual(called, [HEADER])
         self.assertEqual(protocol.state, STATUS)
 
@@ -243,7 +250,7 @@ class HTTPParserTests(TestCase):
         C{headerReceived}.
         """
         header, protocol = self._headerTestSetup()
-        protocol.dataReceived('\r\n')
+        protocol.dataReceived(self.sep)
         self.assertEqual(header, {})
         self.assertEqual(protocol.state, BODY)
 
@@ -255,13 +262,12 @@ class HTTPParserTests(TestCase):
         """
         protocol = HTTPParser()
         protocol.makeConnection(StringTransport())
-        protocol.dataReceived('HTTP/1.1 200 OK\r\n')
-        protocol.dataReceived('X-Foo: bar\r\n')
-        protocol.dataReceived('X-Foo: baz\r\n')
-        protocol.dataReceived('\r\n')
-        self.assertEqual(
-            list(protocol.headers.getAllRawHeaders()),
-            [('X-Foo', ['bar', 'baz'])])
+        protocol.dataReceived('HTTP/1.1 200 OK' + self.sep)
+        protocol.dataReceived('X-Foo: bar' + self.sep)
+        protocol.dataReceived('X-Foo: baz' + self.sep)
+        protocol.dataReceived(self.sep)
+        expected = [('X-Foo', ['bar', 'baz'])]
+        self.assertEqual(expected, list(protocol.headers.getAllRawHeaders()))
 
 
     def test_connectionControlHeaders(self):
@@ -298,6 +304,22 @@ class HTTPParserTests(TestCase):
 
 
 
+class HTTPParserTestsRFCComplaintDelimeter(_HTTPParserTests, TestCase):
+    """
+    L{_HTTPParserTests} using standard CR LF newlines.
+    """
+    sep = '\r\n'
+
+
+
+class HTTPParserTestsNonRFCComplaintDelimeter(_HTTPParserTests, TestCase):
+    """
+    L{_HTTPParserTests} using bare LF newlines.
+    """
+    sep = '\n'
+
+
+
 class HTTPClientParserTests(TestCase):
     """
     Tests for L{HTTPClientParser} which is responsible for parsing HTTP
@@ -735,7 +757,8 @@ class HTTPClientParserTests(TestCase):
         is logged and not re-raised.
         """
         transport = StringTransport()
-        protocol = HTTPClientParser(Request('GET', '/', _boringHeaders, None), None)
+        protocol = HTTPClientParser(Request('GET', '/', _boringHeaders, None),
+                                    None)
         protocol.makeConnection(transport)
 
         response = []
@@ -936,6 +959,27 @@ class HTTP11ClientProtocolTests(TestCase):
         return self.test_connectionLostDuringRequestGeneration('errback')
 
 
+    def test_errorMessageOnConnectionLostBeforeGenerationFailedDoesNotConfuse(self):
+        """
+        If the request passed to L{HTTP11ClientProtocol} finished generation
+        with an error after the L{HTTP11ClientProtocol}'s connection has been
+        lost, an error is logged that gives a non-confusing hint to user on what
+        went wrong.
+        """
+        errors = []
+        log.addObserver(errors.append)
+        self.addCleanup(log.removeObserver, errors.append)
+
+        def check(ignore):
+            error = errors[0]
+            self.assertEquals(error['why'],
+                              'Error writing request, but not in valid state '
+                              'to finalize request: CONNECTION_LOST')
+
+        return self.test_connectionLostDuringRequestGeneration(
+            'errback').addCallback(check)
+
+
     def test_receiveSimplestResponse(self):
         """
         When a response is delivered to L{HTTP11ClientProtocol}, the
@@ -1150,6 +1194,90 @@ class HTTP11ClientProtocolTests(TestCase):
         return assertResponseFailed(self, requestDeferred, [ConnectionDone])
 
 
+    def test_abortClosesConnection(self):
+        """
+        The transport will be told to close its connection when
+        L{HTTP11ClientProtocol.abort} is invoked.
+        """
+        transport = StringTransport()
+        protocol = HTTP11ClientProtocol()
+        protocol.makeConnection(transport)
+        protocol.abort()
+        self.assertTrue(transport.disconnecting)
+
+
+    def test_abortBeforeResponseBody(self):
+        """
+        The Deferred returned by L{HTTP11ClientProtocol.request} will fire
+        with a L{ResponseFailed} failure containing a L{ConnectionAborted}
+        exception, if the connection was aborted before all response headers
+        have been received.
+        """
+        transport = StringTransport()
+        protocol = HTTP11ClientProtocol()
+        protocol.makeConnection(transport)
+        result = protocol.request(Request('GET', '/', _boringHeaders, None))
+        protocol.abort()
+        self.assertTrue(transport.disconnecting)
+        protocol.connectionLost(Failure(ConnectionDone()))
+        return assertResponseFailed(self, result, [ConnectionAborted])
+
+
+    def test_abortAfterResponseHeaders(self):
+        """
+        When the connection is aborted after the response headers have
+        been received and the L{Response} has been made available to
+        application code, the response body protocol's C{connectionLost}
+        method will be invoked with a L{ResponseFailed} failure containing a
+        L{ConnectionAborted} exception.
+        """
+        transport = StringTransport()
+        protocol = HTTP11ClientProtocol()
+        protocol.makeConnection(transport)
+        result = protocol.request(Request('GET', '/', _boringHeaders, None))
+
+        protocol.dataReceived(
+            "HTTP/1.1 200 OK\r\n"
+            "Content-Length: 1\r\n"
+            "\r\n"
+            )
+
+        testResult = Deferred()
+
+        class BodyDestination(Protocol):
+            """
+            A body response protocol which immediately aborts the HTTP
+            connection.
+            """
+            def connectionMade(self):
+                """
+                Abort the HTTP connection.
+                """
+                protocol.abort()
+
+            def connectionLost(self, reason):
+                """
+                Make the reason for the losing of the connection available to
+                the unit test via C{testResult}.
+                """
+                testResult.errback(reason)
+
+
+        def deliverBody(response):
+            """
+            Connect the L{BodyDestination} response body protocol to the
+            response, and then simulate connection loss after ensuring that
+            the HTTP connection has been aborted.
+            """
+            response.deliverBody(BodyDestination())
+            self.assertTrue(transport.disconnecting)
+            protocol.connectionLost(Failure(ConnectionDone()))
+
+        result.addCallback(deliverBody)
+        return assertResponseFailed(self, testResult,
+                                    [ConnectionAborted, _DataLoss])
+
+
 
 class StringProducer:
     """
