@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -309,22 +309,6 @@ class StaticFileTests(TestCase):
         return d
 
 
-    def test_createPickleChild(self):
-        """
-        L{static.File.createPickleChild} is deprecated.
-        """
-        path = FilePath(self.mktemp())
-        path.makedirs()
-        static.File(path.path).createPickleChild("foo", None)
-        warnings = self.flushWarnings([self.test_createPickleChild])
-        self.assertEqual(warnings[0]['category'], DeprecationWarning)
-        self.assertEqual(
-            warnings[0]['message'],
-            "File.createPickleChild is deprecated since Twisted 9.0.  "
-            "Resource persistence is beyond the scope of Twisted Web.")
-        self.assertEqual(len(warnings), 1)
-
-
 
 class StaticMakeProducerTests(TestCase):
     """
@@ -1376,6 +1360,20 @@ class DirectoryListerTest(TestCase):
         self.assertTrue(content[4].startswith('<tr class="odd">'))
 
 
+    def test_contentType(self):
+        """
+        L{static.DirectoryLister} produces a MIME-type that indicates that it is
+        HTML, and includes its charset (UTF-8).
+        """
+        path = FilePath(self.mktemp())
+        path.makedirs()
+        lister = static.DirectoryLister(path.path)
+        req = self._request('')
+        lister.render(req)
+        self.assertEquals(req.outgoingHeaders['content-type'],
+                          "text/html; charset=utf-8")
+
+
     def test_mimeTypeAndEncodings(self):
         """
         L{static.DirectoryLister} is able to detect mimetype and encoding of
