@@ -1,4 +1,4 @@
-# Copyright (c) 2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -9,19 +9,22 @@ __metaclass__ = type
 
 from sys import exc_info
 from urllib import quote
-from StringIO import StringIO
 from thread import get_ident
+import StringIO, cStringIO, tempfile
 
 from zope.interface.verify import verifyObject
 
 from twisted.python.compat import set
-from twisted.python.log import addObserver, removeObserver
+from twisted.python.log import addObserver, removeObserver, err
+from twisted.python.failure import Failure
 from twisted.python.threadpool import ThreadPool
 from twisted.internet.defer import Deferred, gatherResults
 from twisted.internet import reactor
+from twisted.internet.error import ConnectionLost
 from twisted.trial.unittest import TestCase
+from twisted.web import http
 from twisted.web.resource import IResource, Resource
-from twisted.web.server import Request, Site
+from twisted.web.server import Request, Site, version
 from twisted.web.wsgi import WSGIResource
 from twisted.web.test.test_web import DummyChannel
 
@@ -38,7 +41,13 @@ class SynchronousThreadPool:
         Call C{f(*a, **kw)} in this thread rather than scheduling it to be
         called in a thread.
         """
-        f(*a, **kw)
+        try:
+            f(*a, **kw)
+        except:
+            # callInThread doesn't let exceptions propagate to the caller.
+            # None is always returned and any exception raised gets logged
+            # later on.
+            err(None, "Callable passed to SynchronousThreadPool.callInThread failed")
 
 
 
@@ -153,7 +162,8 @@ class WSGITestsMixin:
             request.requestHeaders.addRawHeader(k, v)
         request.gotLength(0)
         if body:
-            request.content = StringIO(body)
+            request.content.write(body)
+            request.content.seek(0)
         uri = '/' + '/'.join([quote(seg, safe) for seg in requestSegments])
         if query is not None:
             uri += '?' + '&'.join(['='.join([quote(k, safe), quote(v, safe)])
@@ -241,7 +251,7 @@ class EnvironTests(WSGITestsMixin, TestCase):
         resource (RFC 3875, section 4.1.13).
         """
         root = self.render('GET', '1.1', [], [''])
-        root.addCallback(self.environKeyEqual('SCRIPT_NAME', '/'))
+        root.addCallback(self.environKeyEqual('SCRIPT_NAME', ''))
 
         emptyChild = self.render('GET', '1.1', [''], [''])
         emptyChild.addCallback(self.environKeyEqual('SCRIPT_NAME', '/'))
@@ -276,7 +286,7 @@ class EnvironTests(WSGITestsMixin, TestCase):
         assertKeyEmpty = self.environKeyEqual('PATH_INFO', '')
 
         root = self.render('GET', '1.1', [], [''])
-        root.addCallback(assertKeyEmpty)
+        root.addCallback(self.environKeyEqual('PATH_INFO', '/'))
 
         emptyChild = self.render('GET', '1.1', [''], [''])
         emptyChild.addCallback(assertKeyEmpty)
@@ -318,11 +328,11 @@ class EnvironTests(WSGITestsMixin, TestCase):
         present.addCallback(self.environKeyEqual('QUERY_STRING', 'foo=bar'))
 
         unencoded = self.render('GET', '1.1', [], [''], [('/', '/')])
-        unencoded.addCallback(self.environKeyEqual('QUERY_STRING', '/=/'))
+        unencoded.addCallback(self.environKeyEqual('QUERY_STRING', '%2F=%2F'))
 
         # "?" is reserved in the <searchpart> portion of a URL.  However, it
-        # seems to be a common mistake of clients to forget to quote it. 
-        # So, make sure we handle that invalid case.
+        # seems to be a common mistake of clients to forget to quote it.  So,
+        # make sure we handle that invalid case.
         doubleQuestion = self.render(
             'GET', '1.1', [], [''], [('foo', '?bar')], safe='?')
         doubleQuestion.addCallback(
@@ -418,6 +428,16 @@ class EnvironTests(WSGITestsMixin, TestCase):
         return gatherResults([old, new])
 
 
+    def test_remoteAddr(self):
+        """
+        The C{'REMOTE_ADDR'} key of the C{environ} C{dict} passed to the
+        application contains the address of the client making the request.
+        """
+        d = self.render('GET', '1.1', [], [''])
+        d.addCallback(self.environKeyEqual('REMOTE_ADDR', '192.168.1.1'))
+
+        return d
+
     def test_headers(self):
         """
         HTTP request headers are copied into the C{environ} C{dict} passed to
@@ -451,9 +471,9 @@ class EnvironTests(WSGITestsMixin, TestCase):
         application has the value C{(1, 0)} indicating that this is a WSGI 1.0
         container.
         """
-        version = self.render('GET', '1.1', [], [''])
-        version.addCallback(self.environKeyEqual('wsgi.version', (1, 0)))
-        return version
+        versionDeferred = self.render('GET', '1.1', [], [''])
+        versionDeferred.addCallback(self.environKeyEqual('wsgi.version', (1, 0)))
+        return versionDeferred
 
 
     def test_wsgiRunOnce(self):
@@ -499,14 +519,14 @@ class EnvironTests(WSGITestsMixin, TestCase):
             return channel
 
         self.channelFactory = DummyChannel
-        http = self.render('GET', '1.1', [], [''])
-        http.addCallback(self.environKeyEqual('wsgi.url_scheme', 'http'))
+        httpDeferred = self.render('GET', '1.1', [], [''])
+        httpDeferred.addCallback(self.environKeyEqual('wsgi.url_scheme', 'http'))
 
         self.channelFactory = channelFactory
-        https = self.render('GET', '1.1', [], [''])
-        https.addCallback(self.environKeyEqual('wsgi.url_scheme', 'https'))
+        httpsDeferred = self.render('GET', '1.1', [], [''])
+        httpsDeferred.addCallback(self.environKeyEqual('wsgi.url_scheme', 'https'))
 
-        return gatherResults([http, https])
+        return gatherResults([httpDeferred, httpsDeferred])
 
 
     def test_wsgiErrors(self):
@@ -538,14 +558,26 @@ class EnvironTests(WSGITestsMixin, TestCase):
         return errors
 
 
-    def test_wsgiInput(self):
-        """
-        The C{'wsgi.input'} key of the C{environ} C{dict} passed to the
-        application is a file-like object (as defined in the U{Input and Errors
-        Streams<http://www.python.org/dev/peps/pep-0333/#input-and-error-streams>}
-        section of PEP 333) which makes the request body available to the
-        application.
-        """
+class InputStreamTestMixin(WSGITestsMixin):
+    """
+    A mixin for L{TestCase} subclasses which defines a number of tests against
+    L{_InputStream}.  The subclass is expected to create a file-like object to
+    be wrapped by an L{_InputStream} under test.
+    """
+    def getFileType(self):
+        raise NotImplementedError(
+            "%s.getFile must be implemented" % (self.__class__.__name__,))
+
+
+    def _renderAndReturnReaderResult(self, reader, content):
+        contentType = self.getFileType()
+        class CustomizedRequest(Request):
+            def gotLength(self, length):
+                # Always allocate a file of the specified type, instead of
+                # using the base behavior of selecting one depending on the
+                # length.
+                self.content = contentType()
+
         def appFactoryFactory(reader):
             result = Deferred()
             def applicationFactory():
@@ -556,88 +588,302 @@ class EnvironTests(WSGITestsMixin, TestCase):
                     return iter(())
                 return application
             return result, applicationFactory
-
-        inputRead, appFactory = appFactoryFactory(
-            lambda input: [input.read(1), input.read()])
+        d, appFactory = appFactoryFactory(reader)
         self.lowLevelRender(
-            Request, appFactory, DummyChannel,
-            'GET', '1.1', [], [''], None, [],
-            "hello, world\n"
-            "how are you\n")
-        inputRead.addCallback(
-            self.assertEqual, ['h', 'ello, world\nhow are you\n'])
-
-        # COMPATIBILITY NOTE: the size argument is excluded from the WSGI
-        # specification, but is provided here anyhow, because useful libraries
-        # such as python stdlib's cgi.py assume their input file-like-object
-        # supports readline with a size argument. If you use it, be aware your
-        # application may not be portable to other conformant WSGI servers.
-        inputReadline, appFactory = appFactoryFactory(
-            lambda input: [input.readline(), input.readline(None),
-                           input.readline(-1), input.readline(20),
-                           input.readline(5), input.readline(5),
-                           input.readline(5)])
-        self.lowLevelRender(
-            Request, appFactory, DummyChannel,
-            'GET', '1.1', [], [''], None, [],
-            "hello, world\n"
-            "how are you\n"
-            "I am great\n"
-            "goodbye now\n"
-            "no data here\n")
-        inputReadline.addCallback(
-            self.assertEqual, [
-                'hello, world\n', 'how are you\n', 'I am great\n',
-                'goodbye now\n', 'no da', 'ta he', 're\n'])
+            CustomizedRequest, appFactory, DummyChannel,
+            'PUT', '1.1', [], [''], None, [],
+            content)
+        return d
 
 
-        inputReadlinesNoArg, appFactory = appFactoryFactory(
-            lambda input: input.readlines())
-        self.lowLevelRender(
-            Request, appFactory, DummyChannel,
-            'GET', '1.1', [], [''], None, [],
-            "foo\n"
-            "bar\n")
-        inputReadlinesNoArg.addCallback(
-            self.assertEqual, ["foo\n", "bar\n"])
+    def test_readAll(self):
+        """
+        Calling L{_InputStream.read} with no arguments returns the entire input
+        stream.
+        """
+        bytes = "some bytes are here"
+        d = self._renderAndReturnReaderResult(lambda input: input.read(), bytes)
+        d.addCallback(self.assertEquals, bytes)
+        return d
 
 
-        inputReadlinesNone, appFactory = appFactoryFactory(
-            lambda input: input.readlines(None))
-        self.lowLevelRender(
-            Request, appFactory, DummyChannel,
-            'GET', '1.1', [], [''], None, [],
-            "foo\n"
-            "bar\n")
-        inputReadlinesNone.addCallback(
-            self.assertEqual, ["foo\n", "bar\n"])
+    def test_readSome(self):
+        """
+        Calling L{_InputStream.read} with an integer returns that many bytes
+        from the input stream, as long as it is less than or equal to the total
+        number of bytes available.
+        """
+        bytes = "hello, world."
+        d = self._renderAndReturnReaderResult(lambda input: input.read(3), bytes)
+        d.addCallback(self.assertEquals, "hel")
+        return d
 
 
-        inputReadlinesLength, appFactory = appFactoryFactory(
-            lambda input: input.readlines(6))
-        self.lowLevelRender(
-            Request, appFactory, DummyChannel,
-            'GET', '1.1', [], [''], None, [],
-            "foo\n"
-            "bar\n")
-        inputReadlinesLength.addCallback(
-            self.assertEqual, ["foo\n", "bar\n"])
+    def test_readMoreThan(self):
+        """
+        Calling L{_InputStream.read} with an integer that is greater than the
+        total number of bytes in the input stream returns all bytes in the
+        input stream.
+        """
+        bytes = "some bytes are here"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.read(len(bytes) + 3), bytes)
+        d.addCallback(self.assertEquals, bytes)
+        return d
 
 
-        inputIter, appFactory = appFactoryFactory(
-            lambda input: list(input))
-        self.lowLevelRender(
-            Request, appFactory, DummyChannel,
-            'GET', '1.1', [], [''], None, [],
-            'foo\n'
-            'bar\n')
-        inputIter.addCallback(
-            self.assertEqual, ['foo\n', 'bar\n'])
+    def test_readTwice(self):
+        """
+        Calling L{_InputStream.read} a second time returns bytes starting from
+        the position after the last byte returned by the previous read.
+        """
+        bytes = "some bytes, hello"
+        def read(input):
+            input.read(3)
+            return input.read()
+        d = self._renderAndReturnReaderResult(read, bytes)
+        d.addCallback(self.assertEquals, bytes[3:])
+        return d
 
-        return gatherResults([
-                inputRead, inputReadline, inputReadlinesNoArg,
-                inputReadlinesNone, inputReadlinesLength,
-                inputIter])
+
+    def test_readNone(self):
+        """
+        Calling L{_InputStream.read} with C{None} as an argument returns all
+        bytes in the input stream.
+        """
+        bytes = "the entire stream"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.read(None), bytes)
+        d.addCallback(self.assertEquals, bytes)
+        return d
+
+
+    def test_readNegative(self):
+        """
+        Calling L{_InputStream.read} with a negative integer as an argument
+        returns all bytes in the input stream.
+        """
+        bytes = "all of the input"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.read(-1), bytes)
+        d.addCallback(self.assertEquals, bytes)
+        return d
+
+
+    def test_readline(self):
+        """
+        Calling L{_InputStream.readline} with no argument returns one line from
+        the input stream.
+        """
+        bytes = "hello\nworld"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.readline(), bytes)
+        d.addCallback(self.assertEquals, "hello\n")
+        return d
+
+
+    def test_readlineSome(self):
+        """
+        Calling L{_InputStream.readline} with an integer returns at most that
+        many bytes, even if it is not enough to make up a complete line.
+
+        COMPATIBILITY NOTE: the size argument is excluded from the WSGI
+        specification, but is provided here anyhow, because useful libraries
+        such as python stdlib's cgi.py assume their input file-like-object
+        supports readline with a size argument. If you use it, be aware your
+        application may not be portable to other conformant WSGI servers.
+        """
+        bytes = "goodbye\nworld"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.readline(3), bytes)
+        d.addCallback(self.assertEquals, "goo")
+        return d
+
+
+    def test_readlineMoreThan(self):
+        """
+        Calling L{_InputStream.readline} with an integer which is greater than
+        the number of bytes in the next line returns only the next line.
+        """
+        bytes = "some lines\nof text"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.readline(20), bytes)
+        d.addCallback(self.assertEquals, "some lines\n")
+        return d
+
+
+    def test_readlineTwice(self):
+        """
+        Calling L{_InputStream.readline} a second time returns the line
+        following the line returned by the first call.
+        """
+        bytes = "first line\nsecond line\nlast line"
+        def readline(input):
+            input.readline()
+            return input.readline()
+        d = self._renderAndReturnReaderResult(readline, bytes)
+        d.addCallback(self.assertEquals, "second line\n")
+        return d
+
+
+    def test_readlineNone(self):
+        """
+        Calling L{_InputStream.readline} with C{None} as an argument returns
+        one line from the input stream.
+        """
+        bytes = "this is one line\nthis is another line"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.readline(None), bytes)
+        d.addCallback(self.assertEquals, "this is one line\n")
+        return d
+
+
+    def test_readlineNegative(self):
+        """
+        Calling L{_InputStream.readline} with a negative integer as an argument
+        returns one line from the input stream.
+        """
+        bytes = "input stream line one\nline two"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.readline(-1), bytes)
+        d.addCallback(self.assertEquals, "input stream line one\n")
+        return d
+
+
+    def test_readlines(self):
+        """
+        Calling L{_InputStream.readlines} with no arguments returns a list of
+        all lines from the input stream.
+        """
+        bytes = "alice\nbob\ncarol"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.readlines(), bytes)
+        d.addCallback(self.assertEquals, ["alice\n", "bob\n", "carol"])
+        return d
+
+
+    def test_readlinesSome(self):
+        """
+        Calling L{_InputStream.readlines} with an integer as an argument
+        returns a list of lines from the input stream with the argument serving
+        as an approximate bound on the total number of bytes to read.
+        """
+        bytes = "123\n456\n789\n0"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.readlines(5), bytes)
+        def cbLines(lines):
+            # Make sure we got enough lines to make 5 bytes.  Anything beyond
+            # that is fine too.
+            self.assertEquals(lines[:2], ["123\n", "456\n"])
+        d.addCallback(cbLines)
+        return d
+
+
+    def test_readlinesMoreThan(self):
+        """
+        Calling L{_InputStream.readlines} with an integer which is greater than
+        the total number of bytes in the input stream returns a list of all
+        lines from the input.
+        """
+        bytes = "one potato\ntwo potato\nthree potato"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.readlines(100), bytes)
+        d.addCallback(
+            self.assertEquals,
+            ["one potato\n", "two potato\n", "three potato"])
+        return d
+
+
+    def test_readlinesAfterRead(self):
+        """
+        Calling L{_InputStream.readlines} after a call to L{_InputStream.read}
+        returns lines starting at the byte after the last byte returned by the
+        C{read} call.
+        """
+        bytes = "hello\nworld\nfoo"
+        def readlines(input):
+            input.read(7)
+            return input.readlines()
+        d = self._renderAndReturnReaderResult(readlines, bytes)
+        d.addCallback(self.assertEquals, ["orld\n", "foo"])
+        return d
+
+
+    def test_readlinesNone(self):
+        """
+        Calling L{_InputStream.readlines} with C{None} as an argument returns
+        all lines from the input.
+        """
+        bytes = "one fish\ntwo fish\n"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.readlines(None), bytes)
+        d.addCallback(self.assertEquals, ["one fish\n", "two fish\n"])
+        return d
+
+
+    def test_readlinesNegative(self):
+        """
+        Calling L{_InputStream.readlines} with a negative integer as an
+        argument returns a list of all lines from the input.
+        """
+        bytes = "red fish\nblue fish\n"
+        d = self._renderAndReturnReaderResult(
+            lambda input: input.readlines(-1), bytes)
+        d.addCallback(self.assertEquals, ["red fish\n", "blue fish\n"])
+        return d
+
+
+    def test_iterable(self):
+        """
+        Iterating over L{_InputStream} produces lines from the input stream.
+        """
+        bytes = "green eggs\nand ham\n"
+        d = self._renderAndReturnReaderResult(lambda input: list(input), bytes)
+        d.addCallback(self.assertEquals, ["green eggs\n", "and ham\n"])
+        return d
+
+
+    def test_iterableAfterRead(self):
+        """
+        Iterating over L{_InputStream} after calling L{_InputStream.read}
+        produces lines from the input stream starting from the first byte after
+        the last byte returned by the C{read} call.
+        """
+        bytes = "green eggs\nand ham\n"
+        def iterate(input):
+            input.read(3)
+            return list(input)
+        d = self._renderAndReturnReaderResult(iterate, bytes)
+        d.addCallback(self.assertEquals, ["en eggs\n", "and ham\n"])
+        return d
+
+
+
+class InputStreamStringIOTests(InputStreamTestMixin, TestCase):
+    """
+    Tests for L{_InputStream} when it is wrapped around a L{StringIO.StringIO}.
+    """
+    def getFileType(self):
+        return StringIO.StringIO
+
+
+
+class InputStreamCStringIOTests(InputStreamTestMixin, TestCase):
+    """
+    Tests for L{_InputStream} when it is wrapped around a
+    L{cStringIO.StringIO}.
+    """
+    def getFileType(self):
+        return cStringIO.StringIO
+
+
+
+class InputStreamTemporaryFileTests(InputStreamTestMixin, TestCase):
+    """
+    Tests for L{_InputStream} when it is wrapped around a L{tempfile.TemporaryFile}.
+    """
+    def getFileType(self):
+        return tempfile.TemporaryFile
 
 
 
@@ -673,24 +919,37 @@ class StartResponseTests(WSGITestsMixin, TestCase):
         return d
 
 
-    def test_headers(self):
+    def _headersTest(self, appHeaders, expectedHeaders):
         """
-        The headers passed to the I{start_response} callable are included in
-        the headers response.
+        Verify that if the response headers given by C{appHeaders} are passed
+        to the I{start_response} callable, then the response header lines given
+        by C{expectedHeaders} plus I{Server} and I{Date} header lines are
+        included in the response.
         """
+        # Make the Date header value deterministic
+        self.patch(http, 'datetimeToString', lambda: 'Tuesday')
+
         channel = DummyChannel()
 
         def applicationFactory():
             def application(environ, startResponse):
-                startResponse('200 OK', [('foo', 'bar'), ('baz', 'quux')])
+                startResponse('200 OK', appHeaders)
                 return iter(())
             return application
 
         d, requestFactory = self.requestFactoryFactory()
         def cbRendered(ignored):
-            responseLines = channel.transport.written.getvalue().split('\r\n')
-            self.assertIn('Foo: bar', responseLines)
-            self.assertIn('Baz: quux', responseLines)
+            response = channel.transport.written.getvalue()
+            headers, rest = response.split('\r\n\r\n', 1)
+            headerLines = headers.split('\r\n')[1:]
+            headerLines.sort()
+            allExpectedHeaders = expectedHeaders + [
+                'Date: Tuesday',
+                'Server: ' + version,
+                'Transfer-Encoding: chunked']
+            allExpectedHeaders.sort()
+            self.assertEqual(headerLines, allExpectedHeaders)
+
         d.addCallback(cbRendered)
 
         request = self.lowLevelRender(
@@ -699,6 +958,39 @@ class StartResponseTests(WSGITestsMixin, TestCase):
         return d
 
 
+    def test_headers(self):
+        """
+        The headers passed to the I{start_response} callable are included in
+        the response as are the required I{Date} and I{Server} headers and the
+        necessary connection (hop to hop) header I{Transfer-Encoding}.
+        """
+        return self._headersTest(
+            [('foo', 'bar'), ('baz', 'quux')],
+            ['Baz: quux', 'Foo: bar'])
+
+
+    def test_applicationProvidedContentType(self):
+        """
+        If I{Content-Type} is included in the headers passed to the
+        I{start_response} callable, one I{Content-Type} header is included in
+        the response.
+        """
+        return self._headersTest(
+            [('content-type', 'monkeys are great')],
+            ['Content-Type: monkeys are great'])
+
+
+    def test_applicationProvidedServerAndDate(self):
+        """
+        If either I{Server} or I{Date} is included in the headers passed to the
+        I{start_response} callable, they are disregarded.
+        """
+        return self._headersTest(
+            [('server', 'foo'), ('Server', 'foo'),
+             ('date', 'bar'), ('dATE', 'bar')],
+            [])
+
+
     def test_delayedUntilReturn(self):
         """
         Nothing is written in response to a request when the I{start_response}
@@ -1123,3 +1415,158 @@ class ApplicationTests(WSGITestsMixin, TestCase):
             'GET', '1.1', [], [''])
 
         return d
+
+
+    def test_connectionClosedDuringIteration(self):
+        """
+        If the request connection is lost while the application object is being
+        iterated, iteration is stopped.
+        """
+        class UnreliableConnection(Request):
+            """
+            This is a request which pretends its connection is lost immediately
+            after the first write is done to it.
+            """
+            def write(self, bytes):
+                self.connectionLost(Failure(ConnectionLost("No more connection")))
+
+        self.badIter = False
+        def appIter():
+            yield "foo"
+            self.badIter = True
+            raise Exception("Should not have gotten here")
+
+        def applicationFactory():
+            def application(environ, startResponse):
+                startResponse('200 OK', [])
+                return appIter()
+            return application
+
+        d, requestFactory = self.requestFactoryFactory(UnreliableConnection)
+        def cbRendered(ignored):
+            self.assertFalse(self.badIter, "Should not have resumed iteration")
+        d.addCallback(cbRendered)
+
+        self.lowLevelRender(
+            requestFactory, applicationFactory, DummyChannel,
+            'GET', '1.1', [], [''])
+
+        return self.assertFailure(d, ConnectionLost)
+
+
+    def _internalServerErrorTest(self, application):
+        channel = DummyChannel()
+
+        def applicationFactory():
+            return application
+
+        d, requestFactory = self.requestFactoryFactory()
+        def cbRendered(ignored):
+            errors = self.flushLoggedErrors(RuntimeError)
+            self.assertEquals(len(errors), 1)
+
+            self.assertTrue(
+                channel.transport.written.getvalue().startswith(
+                    'HTTP/1.1 500 Internal Server Error'))
+        d.addCallback(cbRendered)
+
+        request = self.lowLevelRender(
+            requestFactory, applicationFactory,
+            lambda: channel, 'GET', '1.1', [], [''], None, [])
+
+        return d
+
+
+    def test_applicationExceptionBeforeStartResponse(self):
+        """
+        If the application raises an exception before calling I{start_response}
+        then the response status is I{500} and the exception is logged.
+        """
+        def application(environ, startResponse):
+            raise RuntimeError("This application had some error.")
+        return self._internalServerErrorTest(application)
+
+
+    def test_applicationExceptionAfterStartResponse(self):
+        """
+        If the application calls I{start_response} but then raises an exception
+        before any data is written to the response then the response status is
+        I{500} and the exception is logged.
+        """
+        def application(environ, startResponse):
+            startResponse('200 OK', [])
+            raise RuntimeError("This application had some error.")
+        return self._internalServerErrorTest(application)
+
+
+    def _connectionClosedTest(self, application, responseContent):
+        channel = DummyChannel()
+
+        def applicationFactory():
+            return application
+
+        d, requestFactory = self.requestFactoryFactory()
+
+        # Capture the request so we can disconnect it later on.
+        requests = []
+        def requestFactoryWrapper(*a, **kw):
+            requests.append(requestFactory(*a, **kw))
+            return requests[-1]
+
+        def ebRendered(ignored):
+            errors = self.flushLoggedErrors(RuntimeError)
+            self.assertEquals(len(errors), 1)
+
+            response = channel.transport.written.getvalue()
+            self.assertTrue(response.startswith('HTTP/1.1 200 OK'))
+            # Chunked transfer-encoding makes this a little messy.
+            self.assertIn(responseContent, response)
+        d.addErrback(ebRendered)
+
+        request = self.lowLevelRender(
+            requestFactoryWrapper, applicationFactory,
+            lambda: channel, 'GET', '1.1', [], [''], None, [])
+
+        # By now the connection should be closed.
+        self.assertTrue(channel.transport.disconnected)
+        # Give it a little push to go the rest of the way.
+        requests[0].connectionLost(Failure(ConnectionLost("All gone")))
+
+        return d
+
+
+    def test_applicationExceptionAfterWrite(self):
+        """
+        If the application raises an exception after the response status has
+        already been sent then the connection is closed and the exception is
+        logged.
+        """
+        responseContent = (
+            'Some bytes, triggering the server to start sending the response')
+
+        def application(environ, startResponse):
+            startResponse('200 OK', [])
+            yield responseContent
+            raise RuntimeError("This application had some error.")
+        return self._connectionClosedTest(application, responseContent)
+
+
+    def test_applicationCloseException(self):
+        """
+        If the application returns a closeable iterator and the C{close} method
+        raises an exception when called then the connection is still closed and
+        the exception is logged.
+        """
+        responseContent = 'foo'
+
+        class Application(object):
+            def __init__(self, environ, startResponse):
+                startResponse('200 OK', [])
+
+            def __iter__(self):
+                yield responseContent
+
+            def close(self):
+                raise RuntimeError("This application had some error.")
+
+        return self._connectionClosedTest(Application, responseContent)
