@@ -1,25 +1,87 @@
 # -*- test-case-name: twisted.web.test.test_xmlrpc -*-
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 Tests for  XML-RPC support in L{twisted.web.xmlrpc}.
 """
 
+import datetime
 import xmlrpclib
+from StringIO import StringIO
 
 from twisted.trial import unittest
 from twisted.web import xmlrpc
-from twisted.web.xmlrpc import XMLRPC, addIntrospection, _QueryFactory
+from twisted.web.xmlrpc import (
+    XMLRPC, payloadTemplate, addIntrospection, _QueryFactory, Proxy, withRequest)
 from twisted.web import server, static, client, error, http
 from twisted.internet import reactor, defer
 from twisted.internet.error import ConnectionDone
 from twisted.python import failure
+from twisted.test.proto_helpers import MemoryReactor
+from twisted.web.test.test_web import DummyRequest
+try:
+    import twisted.internet.ssl
+except ImportError:
+    sslSkip = "OpenSSL not present"
+else:
+    sslSkip = None
+
+
+class AsyncXMLRPCTests(unittest.TestCase):
+    """
+    Tests for L{XMLRPC}'s support of Deferreds.
+    """
+    def setUp(self):
+        self.request = DummyRequest([''])
+        self.request.method = 'POST'
+        self.request.content = StringIO(
+            payloadTemplate % ('async', xmlrpclib.dumps(())))
+
+        result = self.result = defer.Deferred()
+        class AsyncResource(XMLRPC):
+            def xmlrpc_async(self):
+                return result
+
+        self.resource = AsyncResource()
+
+
+    def test_deferredResponse(self):
+        """
+        If an L{XMLRPC} C{xmlrpc_*} method returns a L{defer.Deferred}, the
+        response to the request is the result of that L{defer.Deferred}.
+        """
+        self.resource.render(self.request)
+        self.assertEquals(self.request.written, [])
+
+        self.result.callback("result")
+
+        resp = xmlrpclib.loads("".join(self.request.written))
+        self.assertEquals(resp, (('result',), None))
+        self.assertEquals(self.request.finished, 1)
+
+
+    def test_interruptedDeferredResponse(self):
+        """
+        While waiting for the L{Deferred} returned by an L{XMLRPC} C{xmlrpc_*}
+        method to fire, the connection the request was issued over may close.
+        If this happens, neither C{write} nor C{finish} is called on the
+        request.
+        """
+        self.resource.render(self.request)
+        self.request.processingFailed(
+            failure.Failure(ConnectionDone("Simulated")))
+        self.result.callback("result")
+        self.assertEquals(self.request.written, [])
+        self.assertEquals(self.request.finished, 0)
+
 
 
 class TestRuntimeError(RuntimeError):
     pass
 
+
+
 class TestValueError(ValueError):
     pass
 
@@ -81,6 +143,20 @@ class Test(XMLRPC):
         return map[key]
     xmlrpc_dict.help = 'Help for dict.'
 
+    @withRequest
+    def xmlrpc_withRequest(self, request, other):
+        """
+        A method decorated with L{withRequest} which can be called by
+        a test to verify that the request object really is passed as
+        an argument.
+        """
+        return (
+            # as a proof that request is a request
+            request.method +
+            # plus proof other arguments are still passed along
+            ' ' + other)
+
+
     def _getFunction(self, functionPath):
         try:
             return XMLRPC._getFunction(self, functionPath)
@@ -113,6 +189,11 @@ class TestQueryProtocol(xmlrpc.QueryProtocol):
     """
     QueryProtocol for tests that saves headers received inside the factory.
     """
+
+    def connectionMade(self):
+        self.factory.transport = self.transport
+        xmlrpc.QueryProtocol.connectionMade(self)
+
     def handleHeader(self, key, val):
         self.factory.headers[key.lower()] = val
 
@@ -128,6 +209,16 @@ class TestQueryFactory(xmlrpc._QueryFactory):
         xmlrpc._QueryFactory.__init__(self, *args, **kwargs)
 
 
+class TestQueryFactoryCancel(xmlrpc._QueryFactory):
+    """
+    QueryFactory that saves a reference to the
+    L{twisted.internet.interfaces.IConnector} to test connection lost.
+    """
+
+    def startedConnecting(self, connector):
+        self.connector = connector
+
+
 class XMLRPCTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -149,9 +240,17 @@ class XMLRPCTestCase(unittest.TestCase):
         self.factories.append(factory)
         return factory
 
-    def proxy(self):
+    def proxy(self, factory=None):
+        """
+        Return a new xmlrpc.Proxy for the test site created in
+        setUp(), using the given factory as the queryFactory, or
+        self.queryFactory if no factory is provided.
+        """
         p = xmlrpc.Proxy("http://127.0.0.1:%d/" % self.port)
-        p.queryFactory = self.queryFactory
+        if factory is None:
+            p.queryFactory = self.queryFactory
+        else:
+            p.queryFactory = factory
         return p
 
     def test_results(self):
@@ -194,6 +293,23 @@ class XMLRPCTestCase(unittest.TestCase):
         d.addCallback(cb)
         return d
 
+
+    def test_cancel(self):
+        """
+        A deferred from the Proxy can be cancelled, disconnecting
+        the L{twisted.internet.interfaces.IConnector}.
+        """
+        def factory(*args, **kw):
+            factory.f = TestQueryFactoryCancel(*args, **kw)
+            return factory.f
+        d = self.proxy(factory).callRemote('add', 2, 3)
+        self.assertNotEquals(factory.f.connector.state, "disconnected")
+        d.cancel()
+        self.assertEquals(factory.f.connector.state, "disconnected")
+        d = self.assertFailure(d, defer.CancelledError)
+        return d
+
+
     def test_errorGet(self):
         """
         A classic GET on the xml server should return a NOT_ALLOWED.
@@ -254,65 +370,169 @@ class XMLRPCTestCase(unittest.TestCase):
         return d
 
 
+    def test_closeConnectionAfterRequest(self):
+        """
+        The connection to the web server is closed when the request is done.
+        """
+        d = self.proxy().callRemote('echo', '')
+        def responseDone(ignored):
+            [factory] = self.factories
+            self.assertFalse(factory.transport.connected)
+            self.assertTrue(factory.transport.disconnected)
+        return d.addCallback(responseDone)
+
+
+    def test_tcpTimeout(self):
+        """
+        For I{HTTP} URIs, L{xmlrpc.Proxy.callRemote} passes the value it
+        received for the C{connectTimeout} parameter as the C{timeout} argument
+        to the underlying connectTCP call.
+        """
+        reactor = MemoryReactor()
+        proxy = xmlrpc.Proxy("http://127.0.0.1:69", connectTimeout=2.0,
+                             reactor=reactor)
+        proxy.callRemote("someMethod")
+        self.assertEquals(reactor.tcpClients[0][3], 2.0)
+
+
+    def test_sslTimeout(self):
+        """
+        For I{HTTPS} URIs, L{xmlrpc.Proxy.callRemote} passes the value it
+        received for the C{connectTimeout} parameter as the C{timeout} argument
+        to the underlying connectSSL call.
+        """
+        reactor = MemoryReactor()
+        proxy = xmlrpc.Proxy("https://127.0.0.1:69", connectTimeout=3.0,
+                             reactor=reactor)
+        proxy.callRemote("someMethod")
+        self.assertEquals(reactor.sslClients[0][4], 3.0)
+    test_sslTimeout.skip = sslSkip
+
+
 
 class XMLRPCTestCase2(XMLRPCTestCase):
     """
     Test with proxy that doesn't add a slash.
     """
 
-    def proxy(self):
+    def proxy(self, factory=None):
         p = xmlrpc.Proxy("http://127.0.0.1:%d" % self.port)
-        p.queryFactory = self.queryFactory
+        if factory is None:
+            p.queryFactory = self.queryFactory
+        else:
+            p.queryFactory = factory
         return p
 
 
-
-class XMLRPCAllowNoneTestCase(unittest.TestCase):
+class SerializationConfigMixin:
     """
-    Test with allowNone set to True.
+    Mixin which defines a couple tests which should pass when a particular flag
+    is passed to L{XMLRPC}.
 
-    These are not meant to be exhaustive serialization tests, since
-    L{xmlrpclib} does all of the actual serialization work.  They are just
-    meant to exercise a few codepaths to make sure we are calling into
-    xmlrpclib correctly.
+    These are not meant to be exhaustive serialization tests, since L{xmlrpclib}
+    does all of the actual serialization work.  They are just meant to exercise
+    a few codepaths to make sure we are calling into xmlrpclib correctly.
+
+    @ivar flagName: A C{str} giving the name of the flag which must be passed to
+        L{XMLRPC} to allow the tests to pass.  Subclasses should set this.
+
+    @ivar value: A value which the specified flag will allow the serialization
+        of.  Subclasses should set this.
     """
-
     def setUp(self):
+        """
+        Create a new XML-RPC server with C{allowNone} set to C{True}.
+        """
+        kwargs = {self.flagName: True}
         self.p = reactor.listenTCP(
-            0, server.Site(Test(allowNone=True)), interface="127.0.0.1")
+            0, server.Site(Test(**kwargs)), interface="127.0.0.1")
+        self.addCleanup(self.p.stopListening)
         self.port = self.p.getHost().port
+        self.proxy = xmlrpc.Proxy(
+            "http://127.0.0.1:%d/" % (self.port,), **kwargs)
 
 
-    def tearDown(self):
-        return self.p.stopListening()
-
-
-    def proxy(self):
-        return xmlrpc.Proxy("http://127.0.0.1:%d" % (self.port,),
-                            allowNone=True)
-
-
-    def test_deferredNone(self):
+    def test_roundtripValue(self):
         """
-        Test that passing a C{None} as an argument to a remote method and
-        returning a L{Deferred} which fires with C{None} properly passes
-        </nil> over the network if allowNone is set to True.
+        C{self.value} can be round-tripped over an XMLRPC method call/response.
         """
-        d = self.proxy().callRemote('defer', None)
-        d.addCallback(self.assertEquals, None)
+        d = self.proxy.callRemote('defer', self.value)
+        d.addCallback(self.assertEquals, self.value)
         return d
 
 
-    def test_dictWithNoneValue(self):
+    def test_roundtripNestedValue(self):
         """
-        Test that return a C{dict} with C{None} as a value works properly.
+        A C{dict} which contains C{self.value} can be round-tripped over an
+        XMLRPC method call/response.
         """
-        d = self.proxy().callRemote('defer', {'a': None})
-        d.addCallback(self.assertEquals, {'a': None})
+        d = self.proxy.callRemote('defer', {'a': self.value})
+        d.addCallback(self.assertEquals, {'a': self.value})
         return d
 
 
 
+class XMLRPCAllowNoneTestCase(SerializationConfigMixin, unittest.TestCase):
+    """
+    Tests for passing C{None} when the C{allowNone} flag is set.
+    """
+    flagName = "allowNone"
+    value = None
+
+
+try:
+    xmlrpclib.loads(xmlrpclib.dumps(({}, {})), use_datetime=True)
+except TypeError:
+    _datetimeSupported = False
+else:
+    _datetimeSupported = True
+
+
+
+class XMLRPCUseDateTimeTestCase(SerializationConfigMixin, unittest.TestCase):
+    """
+    Tests for passing a C{datetime.datetime} instance when the C{useDateTime}
+    flag is set.
+    """
+    flagName = "useDateTime"
+    value = datetime.datetime(2000, 12, 28, 3, 45, 59)
+
+    if not _datetimeSupported:
+        skip = (
+            "Available version of xmlrpclib does not support datetime "
+            "objects.")
+
+
+
+class XMLRPCDisableUseDateTimeTestCase(unittest.TestCase):
+    """
+    Tests for the C{useDateTime} flag on Python 2.4.
+    """
+    if _datetimeSupported:
+        skip = (
+            "Available version of xmlrpclib supports datetime objects.")
+
+    def test_cannotInitializeWithDateTime(self):
+        """
+        L{XMLRPC} raises L{RuntimeError} if passed C{True} for C{useDateTime}.
+        """
+        self.assertRaises(RuntimeError, XMLRPC, useDateTime=True)
+        self.assertRaises(
+            RuntimeError, Proxy, "http://localhost/", useDateTime=True)
+
+
+    def test_cannotSetDateTime(self):
+        """
+        Setting L{XMLRPC.useDateTime} to C{True} after initialization raises
+        L{RuntimeError}.
+        """
+        xmlrpc = XMLRPC(useDateTime=False)
+        self.assertRaises(RuntimeError, setattr, xmlrpc, "useDateTime", True)
+        proxy = Proxy("http://localhost/", useDateTime=False)
+        self.assertRaises(RuntimeError, setattr, proxy, "useDateTime", True)
+
+
+
 class XMLRPCTestAuthenticated(XMLRPCTestCase):
     """
     Test with authenticated proxy. We run this with the same inout/ouput as
@@ -371,7 +591,7 @@ class XMLRPCTestIntrospection(XMLRPCTestCase):
                  'deferFault', 'dict', 'echo', 'fail', 'fault',
                  'pair', 'system.listMethods',
                  'system.methodHelp',
-                 'system.methodSignature'])
+                 'system.methodSignature', 'withRequest'])
 
         d = self.proxy().callRemote("system.listMethods")
         d.addCallback(cbMethods)
@@ -508,3 +728,27 @@ class TestQueryFactoryParseResponse(unittest.TestCase):
         d = self.queryFactory.deferred
         self.queryFactory.parseResponse(content)
         return self.assertFailure(d, IndexError)
+
+
+
+class XMLRPCTestWithRequest(unittest.TestCase):
+
+    def setUp(self):
+        self.resource = Test()
+
+
+    def test_withRequest(self):
+        """
+        When an XML-RPC method is called and the implementation is
+        decorated with L{withRequest}, the request object is passed as
+        the first argument.
+        """
+        request = DummyRequest('/RPC2')
+        request.method = "POST"
+        request.content = StringIO(xmlrpclib.dumps(("foo",), 'withRequest'))
+        def valid(n, request):
+            data = xmlrpclib.loads(request.written[0])
+            self.assertEquals(data, (('POST foo',), None))
+        d = request.notifyFinish().addCallback(valid, request)
+        self.resource.render_POST(request)
+        return d
